package sun.nio.ch;

class NativeThreadSet
{
    private long[] elts;
    private int used;
    private boolean waitingToEmpty;
    
    NativeThreadSet(int n) {
        this.used = 0;
        this.elts = new long[n];
    }
    
    int add() {
        long current = NativeThread.current();
        if (current == 0L) {
            current = -1L;
        }
        synchronized (this) {
            int i = 0;
            if (this.used >= this.elts.length) {
                int length = this.elts.length;
                long[] elts = new long[length * 2];
                System.arraycopy(this.elts, 0, elts, 0, length);
                this.elts = elts;
                i = length;
            }
            while (i < this.elts.length) {
                if (this.elts[i] == 0L) {
                    this.elts[i] = current;
                    ++this.used;
                    return i;
                }
                ++i;
            }
            assert false;
            return -1;
        }
    }
    
    void remove(int n) {
        synchronized (this) {
            this.elts[n] = 0L;
            --this.used;
            if (this.used == 0 && this.waitingToEmpty) {
                this.notifyAll();
            }
        }
    }
    
    synchronized void signalAndWait() {
        boolean n = false;
        while (this.used > 0) {
            int used = this.used;
            for (int length = this.elts.length, i = 0; i < length; ++i) {
                long n2 = this.elts[i];
                if (n2 != 0L) {
                    if (n2 != -1L) {
                        NativeThread.signal(n2);
                    }
                    if (--used == 0) {
                        break;
                    }
                }
            }
            this.waitingToEmpty = true;
            try {
                this.wait(50L);
            }
            catch (InterruptedException ex) {
                n = true;
            }
            finally {
                this.waitingToEmpty = false;
            }
        }
        if (!(n == 0)) {
            Thread.currentThread().interrupt();
        }
    }
}
